import json
import logging
import os
from typing import Dict, List, Union

from repro.common import TemporaryDirectory
from repro.common.docker import make_volume_map, run_command
from repro.common.io import read_jsonl_file, write_to_jsonl_file
from repro.data.types import MetricsType, SummaryType
from repro.models import Model

logger = logging.getLogger(__name__)


def sentence_split(image: str, inputs: List[str]) -> List[List[str]]:
    """
    Run sentence splitting on the inputs.

    Parameters
    ----------
    image : str
        The name of the SacreROUGE Docker image to use.
    inputs : List[str]
        The texts which should be sentence-split.
    Returns
    -------
    List[List[str]]
        The sentence-split text.
    """
    with TemporaryDirectory() as temp:
        host_input_dir = f"{temp}/input"
        host_output_dir = f"{temp}/output"
        volume_map = make_volume_map(host_input_dir, host_output_dir)
        container_input_dir = volume_map[host_input_dir]
        container_output_dir = volume_map[host_output_dir]

        host_input_file = f"{host_input_dir}/input.txt"
        host_output_file = f"{host_output_dir}/output.txt"
        container_input_file = f"{container_input_dir}/input.txt"
        container_output_file = f"{container_output_dir}/output.txt"
        write_to_jsonl_file(inputs, "text", host_input_file)

        command = (
            f"python sentence_split.py {container_input_file} {container_output_file}"
        )
        os.makedirs(host_output_dir)
        run_command(image, command, volume_map=volume_map)

        outputs = read_jsonl_file(host_output_file)
        sentences = [output["sentences"] for output in outputs]
        return sentences


@Model.register("sacrerouge-rouge")
class SRROUGE(Model):
    def __init__(self, image: str = "sacrerouge"):
        self.image = image

    def _maybe_sentence_split(self, summaries: List[SummaryType]) -> List[List[str]]:
        if any(isinstance(summary, str) for summary in summaries):
            if not all(isinstance(summary, str) for summary in summaries):
                raise Exception(
                    f"Input summaries or references are mixed between strings and lists of strings. "
                    f"All must be of the same type"
                )
            return sentence_split(self.image, summaries)
        else:
            return summaries

    def _maybe_sentence_split_references(
        self, references_list: List[List[SummaryType]]
    ) -> List[List[List[str]]]:
        # Flatten the references into a single list so we can call `_maybe_sentence_split`, then
        # rearrange the output to be parallel to `references_list`
        flat_references = []
        for references in references_list:
            flat_references.extend(references)

        split_references = self._maybe_sentence_split(flat_references)
        split_references_list = []
        index = 0
        for references in references_list:
            split_references_list.append([])
            for _ in references:
                split_references_list[-1].append(split_references[index])
                index += 1
        return split_references_list

    def predict(
        self, summary: SummaryType, references: List[SummaryType], **kwargs
    ) -> MetricsType:
        return self.predict_batch(
            [{"summary": summary, "references": references}], **kwargs
        )

    def predict_batch(
        self, inputs: List[Dict[str, Union[str, List[str]]]], **kwargs
    ) -> MetricsType:
        logger.info(f"Calculating ROUGE for {len(inputs)} inputs")

        summaries = [inp["summary"] for inp in inputs]
        references_list = [inp["references"] for inp in inputs]

        summaries = self._maybe_sentence_split(summaries)
        references_list = self._maybe_sentence_split_references(references_list)

        with TemporaryDirectory() as temp:
            host_input_dir = f"{temp}/input"
            host_output_dir = f"{temp}/output"
            volume_map = make_volume_map(host_input_dir, host_output_dir)
            container_input_dir = volume_map[host_input_dir]
            container_output_dir = volume_map[host_output_dir]

            host_input_file = f"{host_input_dir}/input.jsonl"
            container_input_file = f"{container_input_dir}/input.jsonl"

            os.makedirs(host_input_dir)
            with open(host_input_file, "w") as out:
                for i, (summary, references) in enumerate(
                    zip(summaries, references_list)
                ):
                    summary = {"text": summary}
                    references = [{"text": reference} for reference in references]
                    out.write(
                        json.dumps(
                            {
                                "instance_id": str(i),
                                "summarizer_id": "repro",
                                "summarizer_type": "peer",
                                "summary": summary,
                                "references": references,
                            }
                        )
                        + "\n"
                    )

            host_output_file = f"{host_output_dir}/macro.json"
            container_output_file = f"{container_output_dir}/macro.json"
            command = (
                f"sacrerouge rouge evaluate"
                f"  --input-files {container_input_file}"
                f"  --compute_rouge_l true"
                f"  --dataset-reader reference-based"
                f"  --macro-output-json {container_output_file}"
                f"  --micro-output-jsonl {container_output_dir}/micro.jsonl"
            )
            os.makedirs(host_output_dir)
            run_command(
                self.image, command, volume_map=volume_map, network_disabled=True
            )

            scores = json.load(open(host_output_file, "r"))
            return scores["metrics"]



import json
import logging
import os
from typing import Any, Dict, List, Tuple, Union

from repro.common import TemporaryDirectory, util
from repro.common.docker import make_volume_map, run_command
from repro.common.io import read_jsonl_file
from repro.data.types import MetricsType, TextType
from repro.models import Model

logger = logging.getLogger(__name__)


@Model.register("papineni2002-sentbleu")
class SentBLEU(Model):
    def __init__(
        self,
        image: str = "papineni2002",
    ):
        self.image = image

    def predict(
        self,
        candidate: TextType,
        references: List[TextType],
        **kwargs,
    ) -> MetricsType:
        return self.predict_batch(
            [{"candidate": candidate, "references": references}], **kwargs
        )[0]

    def predict_batch(
        self, inputs: List[Dict[str, Union[TextType, List[TextType]]]], **kwargs
    ) -> Tuple[MetricsType, List[MetricsType]]:
        logger.info(f"Calculating SentBLEU for {len(inputs)} inputs")

        candidates = [inp["candidate"] for inp in inputs]
        references_list = [inp["references"] for inp in inputs]

        # Ensure they are `str`
        candidates = [util.flatten(candidate) for candidate in candidates]
        references_list = [
            [util.flatten(reference) for reference in references]
            for references in references_list
        ]

        with TemporaryDirectory() as temp:
            host_input_dir = f"{temp}/input"
            host_output_dir = f"{temp}/output"
            volume_map = make_volume_map(host_input_dir, host_output_dir)
            container_input_dir = volume_map[host_input_dir]
            container_output_dir = volume_map[host_output_dir]

            host_input_file = f"{host_input_dir}/input.jsonl"
            container_input_file = f"{container_input_dir}/input.jsonl"
            os.makedirs(host_input_dir)
            with open(host_input_file, "w") as out:
                for candidate, references in zip(candidates, references_list):
                    out.write(
                        json.dumps({"candidate": candidate, "references": references})
                        + "\n"
                    )

            host_output_file = f"{host_output_dir}/output.jsonl"
            container_output_file = f"{container_output_dir}/output.jsonl"

            command = (
                f"python sentbleu.py"
                f"  --input-file {container_input_file}"
                f"  --output-file {container_output_file}"
            )

            os.makedirs(host_output_dir)
            run_command(
                self.image, command, volume_map=volume_map, network_disabled=True
            )

            micro_metrics = read_jsonl_file(host_output_file)
            macro_metrics = util.average_dicts(micro_metrics)
            return macro_metrics, micro_metrics


@Model.register("papineni2002-bleu")
class BLEU(Model):
    def __init__(
        self,
        image: str = "papineni2002",
    ):
        self.image = image

    def predict(
        self,
        candidate: TextType,
        references: List[TextType],
        **kwargs,
    ) -> MetricsType:
        return self.predict_batch(
            [{"candidate": candidate, "references": references}], **kwargs
        )[0]

    def predict_batch(
        self, inputs: List[Dict[str, Union[TextType, List[TextType]]]], **kwargs
    ) -> Tuple[MetricsType, List[MetricsType]]:
        logger.info(f"Calculating BLEU for {len(inputs)} inputs")

        candidates = [inp["candidate"] for inp in inputs]
        references_list = [inp["references"] for inp in inputs]

        # Ensure they are `str`
        candidates = [util.flatten(candidate) for candidate in candidates]
        references_list = [
            [util.flatten(reference) for reference in references]
            for references in references_list
        ]

        with TemporaryDirectory() as temp:
            host_input_dir = f"{temp}/input"
            host_output_dir = f"{temp}/output"
            volume_map = make_volume_map(host_input_dir, host_output_dir)
            container_input_dir = volume_map[host_input_dir]
            container_output_dir = volume_map[host_output_dir]

            host_input_file = f"{host_input_dir}/input.jsonl"
            container_input_file = f"{container_input_dir}/input.jsonl"
            os.makedirs(host_input_dir)
            with open(host_input_file, "w") as out:
                for candidate, references in zip(candidates, references_list):
                    out.write(
                        json.dumps({"candidate": candidate, "references": references})
                        + "\n"
                    )

            host_output_file = f"{host_output_dir}/output.json"
            container_output_file = f"{container_output_dir}/output.json"

            command = (
                f"python bleu.py"
                f"  --input-file {container_input_file}"
                f"  --output-file {container_output_file}"
            )

            os.makedirs(host_output_dir)
            run_command(
                self.image, command, volume_map=volume_map, network_disabled=True
            )

            # BLEU is corpus-level, so there's no micro
            macro_metrics = json.load(open(host_output_file, "r"))
            micro_metrics = []
            return macro_metrics, micro_metrics



import json
import logging
from typing import Dict, List, Tuple, Union

from repro.common import util
from repro.common.docker import DockerContainer
from repro.common.io import read_jsonl_file
from repro.data.types import MetricsType, TextType
from repro.models import Model

logger = logging.getLogger(__name__)


@Model.register("durmus2020-feqa")
class FEQA(Model):
    def __init__(self, image: str = "durmus2020", device: int = 0):
        self.image = image
        self.device = device

    @staticmethod
    def _check_sources(sources_list: List[List[TextType]]) -> List[TextType]:
        single_sources = []
        for sources in sources_list:
            if len(sources) != 1:
                raise Exception(
                    f"FEQA only supports single sources. Found: {len(sources)}"
                )
            single_sources.append(sources[0])
        return single_sources

    def predict(
        self,
        candidate: TextType,
        sources: List[TextType],
        **kwargs,
    ) -> MetricsType:
        return self.predict_batch(
            [{"candidate": candidate, "sources": sources}], **kwargs
        )[0]

    def predict_batch(
        self,
        inputs: List[Dict[str, Union[TextType, List[TextType]]]],
        batch_size: int = 16,
        **kwargs,
    ) -> Tuple[MetricsType, List[MetricsType]]:
        logger.info(f"Calculating FEQA for {len(inputs)} inputs")

        candidates = [inp["candidate"] for inp in inputs]
        sources_list = [inp["sources"] for inp in inputs]

        # FEQA only supports single source documents
        sources = self._check_sources(sources_list)

        # Ensure they are all type `str`
        candidates = [util.flatten(candidate) for candidate in candidates]
        sources = [util.flatten(source) for source in sources]

        with DockerContainer(self.image) as backend:
            host_input_file = f"{backend.host_dir}/input.jsonl"
            host_output_file = f"{backend.host_dir}/output.jsonl"
            container_input_file = f"{backend.container_dir}/input.jsonl"
            container_output_file = f"{backend.container_dir}/output.jsonl"
            with open(host_input_file, "w") as out:
                for candidate, source in zip(candidates, sources):
                    out.write(
                        json.dumps(
                            {
                                "candidate": candidate,
                                "source": source,
                            }
                        )
                        + "\n"
                    )

            commands = []
            cuda = self.device != -1
            if cuda:
                commands.append(f"export CUDA_VISIBLE_DEVICES={self.device}")
                score_device = 0
            else:
                score_device = -1

            commands.append("cd feqa")
            commands.append(
                f"python3 score.py"
                f"  --input-file {container_input_file}"
                f"  --cuda-device {score_device}"
                f"  --batch-size {batch_size}"
                f"  --output-file {container_output_file}"
            )

            command = " && ".join(commands)
            backend.run_command(command=command, cuda=cuda, network_disabled=True)

            micro_metrics = read_jsonl_file(host_output_file)
            macro_metrics = util.average_dicts(micro_metrics)
            return macro_metrics, micro_metrics



"""
Modified from https://github.com/neulab/guided_summarization/blob/ea4bbe91f189cdb51f7f6a827210f9adc5319b3c/bart/z_test.py
to allow for CPU, batch size parameter, and tqdm
"""
# fmt: off
import torch
from fairseq.models.bart import GuidedBARTModel
from tqdm import tqdm

import sys
bart = GuidedBARTModel.from_pretrained(
    sys.argv[4],
    checkpoint_file=sys.argv[5],
    data_name_or_path=sys.argv[6]
)

bart.eval()
if torch.cuda.is_available():
    bart.cuda()
    bart.half()

count = 1
bsz = int(sys.argv[7])

with open(sys.argv[1]) as source, open(sys.argv[2]) as zs, open(sys.argv[3], 'w') as fout:
    sline = source.readline().strip()
    slines = [sline]
    zline = zs.readline().strip()
    zlines = [zline]
    for sline, zline in tqdm(zip(source, zs), desc="Running prediction"):
        if count % bsz == 0:
            with torch.no_grad():
                hypotheses_batch = bart.sample(slines, zlines, beam=4, lenpen=2.0, max_len_b=140, min_len=55, no_repeat_ngram_size=3, guided=True)

            for hypothesis in hypotheses_batch:
                fout.write(hypothesis + '\n')
                fout.flush()
            slines = []
            zlines = []

        slines.append(sline.strip())
        zlines.append(zline.strip())
        count += 1
    if slines != []:
        hypotheses_batch = bart.sample(slines, zlines, beam=4, lenpen=2.0, max_len_b=140, min_len=55, no_repeat_ngram_size=3, guided=True)
        for hypothesis in hypotheses_batch:
            fout.write(hypothesis + '\n')
            fout.flush()



from typing import Any, Dict, List, Union

from repro.common import Registrable
from repro.data.types import DocumentType, SummaryType


class Model(Registrable):
    def predict(self, *args, **kwargs):
        """
        Runs inference for the single input instance.
        """
        raise NotImplementedError

    def predict_batch(self, inputs: List[Dict[str, Any]], *args, **kwargs):
        """
        Runs inference for all of the instances in `inputs`. Each item in `inputs`
        should have a key which corresponds to a parameter of the `predict` method.
        For instance, if the signature of predict was:

            `def predict(input1, input2)`

        Then each item in `inputs` should be a dictionary with keys `"input1"` and `"input2"`.
        """
        raise NotImplementedError


class QuestionAnsweringModel(Model):
    def predict(self, context: str, question: str, *args, **kwargs) -> str:
        return self.predict_batch(
            [{"context": context, "question": question}], **kwargs
        )[0]

    def predict_batch(self, inputs: List[Dict[str, str]], *args, **kwargs) -> List[str]:
        raise NotImplementedError


class QuestionGenerationModel(Model):
    def predict(self, context: str, start: int, end: int, **kwargs) -> str:
        return self.predict_batch([{"context": context, "start": start, "end": end}])[0]

    def predict_batch(self, inputs: List[Dict[str, str]], **kwargs) -> List[str]:
        raise NotImplementedError


class RecipeGenerationModel(Model):
    def predict(self, name: str, ingredients: List[str], *args, **kwargs) -> str:
        return self.predict_batch([{"name": name, "ingredients": ingredients}])[0]

    def predict_batch(
        self, inputs: List[Dict[str, Union[str, List[str]]]], *args, **kwargs
    ) -> List[str]:
        raise NotImplementedError


class SingleDocumentSummarizationModel(Model):
    def predict(self, document: DocumentType, *args, **kwargs) -> SummaryType:
        return self.predict_batch([{"document": document}])[0]

    def predict_batch(
        self, inputs: List[Dict[str, DocumentType]], *args, **kwargs
    ) -> List[SummaryType]:
        raise NotImplementedError


class TruecasingModel(Model):
    def predict(self, text: str, *args, **kwargs) -> str:
        return self.predict_batch([{"text": text}])[0]

    def predict_batch(
        self, inputs: List[Dict[str, DocumentType]], *args, **kwargs
    ) -> List[str]:
        raise NotImplementedError



import json
import pytest
import unittest
from parameterized import parameterized

from repro.models.deutsch2021 import (
    QAEval,
    QAEvalQuestionAnsweringModel,
    QAEvalQuestionGenerationModel,
)
from repro.testing import FIXTURES_ROOT as REPRO_FIXTURES_ROOT
from repro.testing import assert_dicts_approx_equal, get_testing_device_parameters

from . import FIXTURES_ROOT


class TestDeutsch2021Models(unittest.TestCase):
    def setUp(self) -> None:
        # These examples were taken from the "qaeval" unit tests
        self.examples = json.load(open(f"{FIXTURES_ROOT}/examples.json", "r"))

        self.multiling2011_examples = json.load(
            open(f"{REPRO_FIXTURES_ROOT}/multiling2011/data.json", "r")
        )

    @parameterized.expand(get_testing_device_parameters())
    def test_question_generation(self, device: int):
        model = QAEvalQuestionGenerationModel(device=device)
        examples = self.examples["generation"]
        inputs = [
            {
                "context": example["context"],
                "start": example["start"],
                "end": example["end"],
            }
            for example in examples
        ]
        expected = [example["question"] for example in examples]
        actual = model.predict_batch(inputs)
        assert actual == expected

    @parameterized.expand(get_testing_device_parameters())
    def test_question_answering(self, device: int):
        model = QAEvalQuestionAnsweringModel(device=device)
        examples = self.examples["answering"]
        inputs = [
            {
                "context": example["context"],
                "question": example["question"],
            }
            for example in examples
        ]
        expected_dicts = [
            {
                "prediction": example["answer"],
                "probability": example["probability"],
                "null_probability": example["null_probability"],
                "start": example["start"],
                "end": example["end"],
            }
            for example in examples
        ]
        expected_answers = [
            example["answer"]
            if example["probability"] > example["null_probability"]
            else None
            for example in examples
        ]
        # Check just the answers
        actual = model.predict_batch(inputs)
        assert actual == expected_answers

        # Check the full dict
        actual = model.predict_batch(inputs, return_dicts=True)
        for a, e in zip(actual, expected_dicts):
            assert a["prediction"] == e["prediction"]
            assert e["null_probability"] == pytest.approx(
                a["null_probability"], abs=1e-4
            )
            assert e["probability"] == pytest.approx(a["probability"], abs=1e-4)
            assert a["start"] == e["start"]
            assert a["end"] == e["end"]

    @parameterized.expand(
        get_testing_device_parameters(gpu_only=True), skip_on_empty=True
    )
    def test_qaeval(self, device: int):
        model = QAEval(device=device)
        inputs = self.multiling2011_examples
        expected_macro = self.examples["metric"]["macro"]
        expected_micro = self.examples["metric"]["micro"]
        actual_macro, actual_micro = model.predict_batch(inputs)

        assert_dicts_approx_equal(expected_macro, actual_macro, abs=1e-4)
        assert len(expected_micro) == len(actual_micro)
        for expected, actual in zip(expected_micro, actual_micro):
            assert_dicts_approx_equal(expected, actual, abs=1e-4)

    @parameterized.expand(
        get_testing_device_parameters(gpu_only=True), skip_on_empty=True
    )
    def test_qaeval_qa_pairs(self, device: int):
        model = QAEval(device=device)
        inputs = [
            {
                "candidate": "Dan walked to the bakery this morning.",
                "references": ["Dan went to buy scones earlier this morning."],
            },
            {
                "candidate": "He bought some scones today",
                "references": ["Dan went to buy scones earlier this morning."],
            },
        ]

        macro, micro, qa_pairs_lists = model.predict_batch(inputs, return_qa_pairs=True)

        assert len(qa_pairs_lists) == 2  # Number of inputs

        qa_pairs_list = qa_pairs_lists[0]
        assert len(qa_pairs_list) == 1  # Number of references
        qa_pairs = qa_pairs_list[0]
        assert len(qa_pairs) == 2
        assert (
            qa_pairs[0]["question"]["question"]
            == "Who went to buy scones earlier this morning?"
        )
        assert qa_pairs[0]["prediction"]["prediction"] == "Dan"
        assert qa_pairs[0]["prediction"]["start"] == 0
        assert qa_pairs[0]["prediction"]["end"] == 3
        assert qa_pairs[0]["prediction"]["is_answered"] == 1.0
        assert qa_pairs[0]["prediction"]["em"] == 1.0
        assert qa_pairs[0]["prediction"]["f1"] == 1.0
        self.assertAlmostEqual(
            qa_pairs[0]["prediction"]["lerc"], 5.035197734832764, places=4
        )
        assert (
            qa_pairs[1]["question"]["question"]
            == "What did Dan go to buy earlier this morning?"
        )
        assert qa_pairs[1]["prediction"]["prediction"] == "bakery"
        assert qa_pairs[1]["prediction"]["start"] == 18
        assert qa_pairs[1]["prediction"]["end"] == 24
        assert qa_pairs[1]["prediction"]["is_answered"] == 1.0
        assert qa_pairs[1]["prediction"]["em"] == 0.0
        assert qa_pairs[1]["prediction"]["f1"] == 0.0
        self.assertAlmostEqual(
            qa_pairs[1]["prediction"]["lerc"], 1.30755615234375, places=4
        )

        qa_pairs_list = qa_pairs_lists[1]
        assert len(qa_pairs_list) == 1  # Number of references
        qa_pairs = qa_pairs_list[0]
        assert len(qa_pairs) == 2
        assert (
            qa_pairs[0]["question"]["question"]
            == "Who went to buy scones earlier this morning?"
        )
        assert qa_pairs[0]["prediction"]["prediction"] == "He"
        assert qa_pairs[0]["prediction"]["start"] == 0
        assert qa_pairs[0]["prediction"]["end"] == 2
        assert qa_pairs[0]["prediction"]["is_answered"] == 0.0
        assert qa_pairs[0]["prediction"]["em"] == 0.0
        assert qa_pairs[0]["prediction"]["f1"] == 0.0
        assert qa_pairs[0]["prediction"]["lerc"] == 0.0
        assert (
            qa_pairs[1]["question"]["question"]
            == "What did Dan go to buy earlier this morning?"
        )
        assert qa_pairs[1]["prediction"]["prediction"] == "scones"
        assert qa_pairs[1]["prediction"]["start"] == 15
        assert qa_pairs[1]["prediction"]["end"] == 21
        assert qa_pairs[1]["prediction"]["is_answered"] == 1.0
        assert qa_pairs[1]["prediction"]["em"] == 1.0
        assert qa_pairs[1]["prediction"]["f1"] == 1.0
        self.assertAlmostEqual(
            qa_pairs[1]["prediction"]["lerc"], 4.984881401062012, places=4
        )



from ._models.chen2020.src.dataset_reader import Chen2020EvaluationDatasetReader
from ._models.chen2020.src.models import LERC, MOCHAEvaluationMetric
from ._models.chen2020.src.setup import Chen2020SetupSubcommand



import json
import unittest

from repro.models.sacrerouge import SRROUGE

from . import FIXTURES_ROOT


class TestSRROUGE(unittest.TestCase):
    def setUp(self) -> None:
        self.image = "sacrerouge"
        self.examples = json.load(open(f"{FIXTURES_ROOT}/examples.json"))

    def test_rouge(self):
        metric = SRROUGE()
        examples = self.examples["rouge"]
        inputs = [
            {"summary": example["summary"], "references": [example["reference"]]}
            for example in examples["input"]
        ]
        expected = examples["output"]
        actual = metric.predict_batch(inputs)
        assert actual == expected



from typing import List

from repro.common import Registrable
from repro.data.types import InstanceDict


class DatasetReader(Registrable):
    def read(self, *input_files: str) -> List[InstanceDict]:
        """
        Loads the instances from the `input_files` by calling the `_read` method.
        After the instances have been loaded, this method verifies that the
        instances all have an "instance_id" key.

        Parameters
        ----------
        input_files : str
            The input files

        Returns
        -------
        List[InstanceDict]
            The instances
        """
        instances = self._read(*input_files)

        # Ensure all instances at least have an instance_id
        for instance in instances:
            if "instance_id" not in instance:
                raise Exception(
                    "All instances returned from a `DatasetReader` are required "
                    "to have an `instance_id` key."
                )
            else:
                instance_id = instance["instance_id"]
                if not isinstance(instance_id, str):
                    raise Exception(
                        f"The value for key `instance_id` is required to be a string. Found: {type(instance_id)}"
                    )

        return instances

    def _read(self, *input_files: str) -> List[InstanceDict]:
        """
        Loads the instances from the `input_files`. Each of the instances should
        have an "instance_id" key with a value that uniquely identifies that instance.

        Parameters
        ----------
        input_files : str
            The input files

        Returns
        -------
        List[InstanceDict]
            The instances
        """
        raise NotImplementedError



import setuptools


# version.py defines the VERSION variable.
# We use exec here so we don't import repro whilst setting up.
VERSION = {}
with open("repro/version.py", "r") as version_file:
    exec(version_file.read(), VERSION)


setuptools.setup(
    name="repro",
    version=VERSION["VERSION"],
    author="Daniel Deutsch",
    description="An open-source library for reproducing results from research papers",
    url="https://github.com/danieldeutsch/repro",
    packages=setuptools.find_packages(exclude=["tests", "tests.*"]),
    entry_points={"console_scripts": ["repro=repro.__main__:main"]},
    python_requires=">=3.6",
    include_package_data=True,
    install_requires=[
        "black==21.7b0",
        "datasets==1.9.0",
        "docker==5.0.0",
        "overrides==3.1.0",
        "parameterized==0.8.1",
        "pytest==6.2.4",
        "six==1.16.0",
    ],
)



import argparse
import json
import os
from allennlp.predictors import Predictor
from typing import Dict, List

from lerc.lerc_predictor import LERCPredictor


def main(args):
    # Loads an AllenNLP Predictor that wraps our trained model
    predictor = Predictor.from_path(
        archive_path=args.model_path,
        predictor_name="lerc",
        cuda_device=args.cuda_device,
    )

    dirname = os.path.dirname(args.output_file)
    if dirname:
        os.makedirs(dirname, exist_ok=True)

    with open(args.output_file, "w") as out:
        with open(args.input_file, "r") as f:
            batch = []
            for line in f:
                data = json.loads(line)
                batch.append(data)

                if len(batch) == args.batch_size:
                    predictions = predictor.predict_batch_json(batch)
                    for prediction in predictions:
                        out.write(json.dumps(prediction) + "\n")
                    batch = []

            if len(batch) > 0:
                predictions = predictor.predict_batch_json(batch)
                for prediction in predictions:
                    out.write(json.dumps(prediction) + "\n")


if __name__ == "__main__":
    argp = argparse.ArgumentParser()
    argp.add_argument("--input-file", required=True)
    argp.add_argument("--model-path", required=True)
    argp.add_argument("--batch-size", required=True, type=int)
    argp.add_argument("--cuda-device", required=True, type=int)
    argp.add_argument("--output-file", required=True)
    args = argp.parse_args()
    main(args)



from ._models.deutsch2021.src.dataset_reader import (
    Deutsch2021QuestionAnsweringEvaluationDatasetReader,
)
from ._models.deutsch2021.src.models import (
    QAEval,
    QAEvalQuestionAnsweringModel,
    QAEvalQuestionGenerationModel,
)
from ._models.deutsch2021.src.setup import Deutsch2021SetupSubcommand



import argparse

from overrides import overrides

from repro import MODELS_ROOT
from repro.commands.subcommand import SetupSubcommand
from repro.common.docker import BuildDockerImageSubcommand, build_image


@SetupSubcommand.register("lewis2020")
class Lewis2020SetupSubcommand(BuildDockerImageSubcommand):
    def __init__(self) -> None:
        super().__init__("lewis2020", f"{MODELS_ROOT}/lewis2020")

    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = f'Build the docker image "{self.image}"'
        self.parser = parser.add_parser(
            self.image, description=description, help=description
        )
        self.parser.add_argument(
            "--cnndm",
            action="store_true",
            help="Indicates the model trained on CNN/DM model should be downloaded",
        )
        self.parser.add_argument(
            "--xsum",
            action="store_true",
            help="Indicates the model trained on XSum should be downloaded",
        )
        self.parser.add_argument(
            "--silent",
            action="store_true",
            help="Silences the output from the build command",
        )
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        build_args = {
            "CNNDM": "true" if args.cnndm else "false",
            "XSUM": "true" if args.xsum else "false",
        }
        build_image(self.root, self.image, build_args=build_args, silent=args.silent)



import json
import os
from pathlib import Path

from repro.common.checks import ConfigurationError
from repro.common.registrable import Registrable
from repro.common.tempdir import TemporaryDirectory

REPRO_CONFIG_FILE = os.getenv(
    "REPRO_CONFIG", default=Path.home() / ".repro/config.json"
)

if os.path.exists(REPRO_CONFIG_FILE):
    config = json.load(open(REPRO_CONFIG_FILE, "r"))
else:
    config = {}

REPRO_CONFIG = {
    "docker_server": config.pop("docker_server", "unix://var/run/docker.sock")
}



import os
import pytest
from typing import List, Tuple

from repro.common import docker
from repro.common.util import NestedDict, flatten_nested_dict

FIXTURES_ROOT = f"{os.path.dirname(os.path.abspath(__file__))}/../tests/fixtures"


def get_testing_device_parameters(gpu_only: bool = False) -> List[Tuple[int]]:
    """
    Gets the list of device parameters to use for unittesting based
    on the value of the TEST_DEVICES environment variable. If `gpu_only`
    is `True`, the CPU will not be returned.
    """
    devices = os.environ.get("TEST_DEVICES", "-1").split(",")
    if gpu_only:
        if "-1" in devices:
            devices.remove("-1")
    return [(int(device),) for device in devices]


def assert_dicts_approx_equal(
    d1: NestedDict, d2: NestedDict, rel: float = None, abs: float = None
):
    d1 = flatten_nested_dict(d1)
    d2 = flatten_nested_dict(d2)
    assert d1.keys() == d2.keys()
    for key in d1.keys():
        assert d1[key] == pytest.approx(d2[key], rel=rel, abs=abs)


class DockerTestContextManager(object):
    def __init__(self, image: str):
        self.image = image

    def __enter__(self):
        # Make sure the image doesn't exist. If it does, delete it
        if docker.image_exists(self.image):
            docker.remove_image(self.image, force=True)

    def __exit__(self, exc_type, exc_val, exc_tb):
        # Make sure the image gets deleted
        if docker.image_exists(self.image):
            docker.remove_image(self.image, force=True)



import json
from overrides import overrides
from typing import List

from repro.data.dataset_readers import DatasetReader
from repro.data.types import InstanceDict


@DatasetReader.register("sacrerouge")
class SacreROUGEDatasetReader(DatasetReader):
    @overrides
    def _read(self, *input_files: str) -> List[InstanceDict]:
        """
        Loads the instances from the `input_files`. Each of the instances should
        have an "instance_id" key with a value that uniquely identifies that instance
        plus a "summary" and "reference" or "references" field.

        Parameters
        ----------
        input_files : str
            The input files

        Returns
        -------
        List[InstanceDict]
            The instances
        """
        instances = []
        for input_file in input_files:
            with open(input_file, "r") as f:
                for line in f:
                    data = json.loads(line)
                    summary = data["summary"]["text"]
                    if "reference" in data:
                        references = [data["reference"]["text"]]
                    else:
                        references = [
                            reference["text"] for reference in data["references"]
                        ]
                    instances.append(
                        {
                            "instance_id": data["instance_id"],
                            "summary": summary,
                            "references": references,
                        }
                    )
        return instances



import json
import unittest
from parameterized import parameterized

from repro.models.gupta2020 import NeuralModuleNetwork
from repro.testing import get_testing_device_parameters

from . import FIXTURES_ROOT


class TestGupta2020(unittest.TestCase):
    def setUp(self) -> None:
        # The examples were taken from the AllenNLP demo for this model
        self.expected_outputs = json.load(open(f"{FIXTURES_ROOT}/expected-output.json"))

    @parameterized.expand(get_testing_device_parameters())
    def test_neural_module_network(self, device: int):
        model = NeuralModuleNetwork(device=device)
        predictions = model.predict_batch(self.expected_outputs)
        answers = [inp["answer"] for inp in self.expected_outputs]
        assert predictions == answers



import os
from typing import List

from repro.common import TemporaryDirectory
from repro.common.docker import make_volume_map, run_command
from repro.common.io import write_to_text_file
from repro.data.types import DocumentType, SummaryType


def sentence_split(image: str, inputs: List[str]) -> List[List[str]]:
    """
    Run sentence splitting on the inputs.

    Parameters
    ----------
    image : str
        The name of the Dou et al. (2021) Docker image to use.
    inputs : List[str]
        The texts which should be sentence-split.

    Returns
    -------
    List[List[str]]
        The sentence-split text.
    """
    with TemporaryDirectory() as temp:
        host_input_dir = f"{temp}/input"
        host_output_dir = f"{temp}/output"
        volume_map = make_volume_map(host_input_dir, host_output_dir)
        container_input_dir = volume_map[host_input_dir]
        container_output_dir = volume_map[host_output_dir]

        host_input_file = f"{host_input_dir}/input.txt"
        host_output_file = f"{host_output_dir}/output.txt"
        container_input_file = f"{container_input_dir}/input.txt"
        container_output_file = f"{container_output_dir}/output.txt"
        write_to_text_file(inputs, host_input_file)

        command = f"python sentence_tokenize.py {container_input_file} {container_output_file}"
        os.makedirs(host_output_dir)
        run_command(image, command, volume_map=volume_map)

        split = [
            line.split("<q>")
            for line in open(host_output_file, "r").read().splitlines()
        ]
        if len(split) != len(inputs):
            raise Exception(
                f"Sentence splitting returned {len(split)} instances but expected {len(inputs)}"
            )
        return split


def get_oracle_sentences(
    image: str, documents: List[List[str]], references: List[List[str]]
) -> List[List[str]]:
    """
    Extracts the oracle sentences from the documents based on the ROUGE
    score calculated against the reference summaries.

    Parameters
    ----------
    image : str
        The name of the Dou et al. (2021) Docker image to use.
    documents : List[List[str]]
        The sentence-split documents
    references : List[List[str]]
        The sentence-split reference summaries

    Returns
    -------
    List[List[str]]
        The oracle sentence-extractive summaries
    """
    with TemporaryDirectory() as temp:
        host_input_dir = f"{temp}/input"
        host_output_dir = f"{temp}/output"
        volume_map = make_volume_map(host_input_dir, host_output_dir)
        container_input_dir = volume_map[host_input_dir]
        container_output_dir = volume_map[host_output_dir]

        host_document_file = f"{host_input_dir}/documents.txt"
        host_reference_file = f"{host_input_dir}/references.txt"
        container_document_file = f"{container_input_dir}/documents.txt"
        container_reference_file = f"{container_input_dir}/references.txt"
        write_to_text_file(documents, host_document_file, separator="<q>")
        write_to_text_file(references, host_reference_file, separator="<q>")

        host_output_file = f"{host_output_dir}/guidance.txt"
        container_output_file = f"{container_output_dir}/guidance.txt"

        commands = [
            f"cd guided_summarization/scripts",
            f"python sents.py {container_document_file} {container_reference_file} {container_output_file}",
        ]
        command = " && ".join(commands)
        os.makedirs(host_output_dir)
        run_command(image, command, volume_map=volume_map)

        guidance = [
            line.split("<q>")
            for line in open(host_output_file, "r").read().splitlines()
        ]
        return guidance


def generate_summaries(
    image: str,
    model: str,
    device: int,
    batch_size: int,
    documents: List[DocumentType],
    guidance: List[SummaryType],
) -> List[str]:
    """
    Generates summaries using a pre-trained model and guidance signals.

    Parameters
    ----------
    image : str
        The name of the Dou et al. (2021) Docker image to use.
    model : str
        The name of the model to use
    device : int
        The GPU ID, -1 for CPU
    batch_size : int
        The batch size
    documents : List[DocumentType]
        The input documents
    guidance : List[SummaryType]
        The guidance signals

    Returns
    -------
    List[str]
        The summaries
    """
    with TemporaryDirectory() as temp:
        host_input_dir = f"{temp}/input"
        host_output_dir = f"{temp}/output"
        volume_map = make_volume_map(host_input_dir, host_output_dir)
        container_input_dir = volume_map[host_input_dir]
        container_output_dir = volume_map[host_output_dir]

        host_document_file = f"{host_input_dir}/documents.txt"
        host_guidance_file = f"{host_input_dir}/guidance.txt"
        container_document_file = f"{container_input_dir}/documents.txt"
        container_guidance_file = f"{container_input_dir}/guidance.txt"
        write_to_text_file(documents, host_document_file)
        write_to_text_file(guidance, host_guidance_file)

        host_output_file = f"{host_output_dir}/summaries.txt"
        container_output_file = f"{container_output_dir}/summaries.txt"

        commands = ["cd guided_summarization/bart"]
        inference_command = (
            f"python summarize.py"
            f"  {container_document_file}"
            f"  {container_guidance_file}"
            f"  {container_output_file}"
            f"  ../{model}"
            f"  model.pt"
            f"  ../{model}"
            f"  {batch_size}"
        )
        cuda = device != -1
        if cuda:
            inference_command = f"CUDA_VISIBLE_DEVICES={device} " + inference_command
        commands.append(inference_command)
        command = " && ".join(commands)

        os.makedirs(host_output_dir)
        run_command(image, command, volume_map=volume_map, cuda=cuda)

        summaries = open(host_output_file, "r").read().splitlines()
        return summaries
